import collections
import hashlib
import subprocess

command_status = collections.namedtuple('command_status',
                                        'stdout stderr retcode')


class mcenv(object):
    def __init__(self, f=["1@480x640"], g=16, i=128):
        if type(f) is str:
            f = [f]

        frame_size = []
        for frame in f:
            frame_size.append('-f')
            frame_size.append(frame)

        graphics_colors = ['-g', str(g)]
        image_colors    = ['-i', str(i)]

        try:
            self.session = subprocess.Popen(['mcenv'] + frame_size +
                                            graphics_colors + image_colors,
                                            bufsize=1, shell=False,
                                            stdin=subprocess.PIPE,
                                            stdout=subprocess.PIPE,
                                            stderr=subprocess.PIPE)
        except OSError:
            raise OSError("mcenv executable not found in PATH environment variable.")

    def command(self, cmd_stdin):
        self.stdin_md5 = hashlib.md5(cmd_stdin).hexdigest()
        self.session.stdin.write("%s\necho -e '%s\n'$?\necho -e '%s' 1>&2\n" %
                                 (cmd_stdin, self.stdin_md5, self.stdin_md5))

        command_stdout = ''
        while True:
            line = self.session.stdout.readline()
            if line.rstrip('\n') == self.stdin_md5:
                break
            else:
                command_stdout += line
        command_stdout = command_stdout.rstrip('\n')

        command_retcode = int(self.session.stdout.readline().rstrip('\n'))

        command_stderr = ''
        while True:
            line = self.session.stderr.readline()
            if line.rstrip('\n') == self.stdin_md5:
                break
            else:
                command_stderr += line
        command_stderr = command_stderr.rstrip('\n')
        return command_status(stdout=command_stdout, stderr=command_stderr,
                              retcode=command_retcode)

    def __getattr__(self, attrname):
        def method(args=''):
            return self.command('%s.k %s' % (attrname, args))
        return method
